@@ -46,10 +46,15 @@ from contextlib import contextmanager
 import msgpack
 import pylibmc
 
-VERSION = "0.4.0"
-PORT = 65447
+VERSION = "0.5.2"
 LOCK_SH = 0
-LOCK_EX = -1
+LOCK_EX = 1
+
+class StarryError(Exception):
+    '''StarryError'''
+
+    def __init__(self, output):
+        self.output = output
 
 def singleton(cls):
     instances = dict()
@@ -173,20 +178,35 @@ class LibMemcachedClient(pylibmc.ThreadMappedPool):
             return mc.delete(key)
 
 class StarryServer(object):
-    def __init__(self, depots=[], port=PORT, log_file=None, rw_ratio=5):
+    def __init__(self, address, depots=[], log_file=None, rw_ratio=5):
+        self.address = address
         self.depots = {}
-        self.port = port
         self.logger = StarryLog(log_file) if log_file is not None else None
         self.rw_ratio = rw_ratio
 
+        if type(self.address) is str:
+            try:
+                os.remove(self.address)
+            except:
+                pass
+
         for depot in depots:
             if "path" in depot:
                 depot_path = os.path.abspath(depot["path"])
 
-                self.depots[depot_path] = StarryDepot(depot_path, self.logger, self.rw_ratio, depot.get("cache", 0))
+                self.depots[depot_path] = StarryDepot(depot_path, self.address, self.logger, self.rw_ratio, depot.get("cache", 0))
+
+    def __del__(self):
+        self.depots.clear()
+
+        if type(self.address) is str:
+            try:
+                os.remove(self.address)
+            except:
+                pass
 
     def start(self, max_workers=512, backlog=512, authkey=b'starry'):
-        with multiprocessing.connection.Listener(address=("0.0.0.0", self.port), backlog=backlog, authkey=authkey) as listener:
+        with multiprocessing.connection.Listener(self.address, backlog=backlog, authkey=authkey) as listener:
             with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
                 while True:
                     try:
@@ -205,7 +225,7 @@ class StarryServer(object):
                         (depot, mode) = conn.recv()
 
                         if depot not in self.depots:
-                            self.depots[depot] = StarryDepot(depot, self.logger, self.rw_ratio)
+                            self.depots[depot] = StarryDepot(depot, self.address, self.logger, self.rw_ratio)
 
                         executor.submit(self.depots[depot].executeCommand, conn, mode)
                     except Exception as e:
@@ -217,7 +237,7 @@ class StarryServer(object):
                             pass
 
 class StarryDepot(object):
-    def __init__(self, path, logger, rw_ratio, cache_size=0):
+    def __init__(self, path, server, logger, rw_ratio, cache_size=0):
         self.database = pathlib.Path(path)
         self.logger = logger
         self.rw_ratio = rw_ratio
@@ -228,19 +248,27 @@ class StarryDepot(object):
         self.read_wait = 0
         self.write_queue = []
         self.read_queue = []
-        self.changes = set()
+
+        if not self.database.is_dir():
+            raise StarryError(f"path {path} doesn't exist.")
 
         self.setCache(cache_size)
 
         if not self.database.joinpath(".backup").is_dir():
             self.database.joinpath(".backup").mkdir(exist_ok=True)
-            self.database.joinpath(".backup", "~.lock").touch()
 
             self.backup()
 
+        self.database.joinpath(".starry").write_bytes(msgpack.packb(server))
+
     def __del__(self):
         self.clearCache()
 
+        try:
+            self.database.joinpath(".starry").unlink()
+        except:
+            pass
+
     def executeCommand(self, conn, mode=LOCK_SH):
         if mode == LOCK_SH:
             while True:
@@ -361,7 +389,7 @@ class StarryDepot(object):
         with self._lock(mark):
             return self._delete(key)
 
-    def clone(self, old_key, new_key):
+    def copy(self, old_key, new_key):
         mark = {"mode": "w", "members": [old_key, new_key]}
 
         with self._lock(mark):
@@ -535,12 +563,12 @@ class StarryDepot(object):
 
             return (True, "")
 
-    def save(self, path):
+    def clone(self, path):
         path = pathlib.Path(path)
 
         if not path.is_dir():
-            self.logger.error(f"No {str(path)}.")
-            return (False, f"No {str(path)}.")
+            self.logger.error(f"path {str(path)} doesn't exist.")
+            return (False, f"path {str(path)} doesn't exist.")
 
         for elem in path.glob("*"):
             if elem.suffix == ".mpb":
@@ -559,54 +587,6 @@ class StarryDepot(object):
 
             return (True, "")
 
-    def load(self, path):
-        path = pathlib.Path(path)
-
-        if not path.is_dir():
-            self.logger.error(f"No {str(path)}.")
-            return (False, f"No {str(path)}.")
-
-        mark = {"mode": "w", "members": [None]}
-
-        with self._lock(mark):
-            for elem in path.glob("*"):
-                if elem.suffix == ".mpb":
-                    (ret, msg) = self._load(elem.stem, path)
-                    if ret is False:
-                        return (ret, msg)
-
-            for elem in self.database.glob("*"):
-                if elem.suffix == ".mpb" and not path.joinpath(elem.name).exists():
-                    self._delete([elem.stem])
-
-            return (True, "")
-
-    def replace(self, path):
-        path = pathlib.Path(path)
-
-        if not path.is_dir():
-            self.logger.error(f"No {str(path)}.")
-            return (False, f"No {str(path)}.")
-
-        mark = {"mode": "w", "members": [None]}
-
-        with self._lock(mark):
-            for elem in path.glob("*"):
-                if elem.suffix == ".mpb":
-                    (ret, msg) = self._load(elem.stem, path)
-                    if ret is False:
-                        return (ret, msg)
-
-            for elem in self.database.glob("*"):
-                if elem.suffix == ".mpb" and not path.joinpath(elem.name).exists():
-                    self._delete([elem.stem])
-
-            for elem in path.glob("*"):
-                if elem.suffix == ".mpb":
-                    elem.unlink()
-
-            return (True, "")
-
     def upgrade(self, path):
         path = pathlib.Path(path)
 
@@ -623,6 +603,10 @@ class StarryDepot(object):
                     if ret is False:
                         return (ret, msg)
 
+            for elem in self.database.glob("*"):
+                if elem.suffix == ".mpb" and not path.joinpath(elem.name).exists():
+                    self._delete([elem.stem])
+
             return (True, "")
 
     def clear(self):
@@ -650,19 +634,6 @@ class StarryDepot(object):
 
         return (True, "")
 
-    def destroy(self):
-        if self.database.is_dir():
-            try:
-                shutil.rmtree(self.database)
-            except Exception as e:
-                self.logger.error(f"Caught exception: {e.__doc__}({e})")
-                return (False, f"Caught exception: {e.__doc__}({e})")
-
-        self.clear()
-        self = []
-
-        return (True, "")
-
     def _getCache(self, key):
         if self.cache is None:
             return None
@@ -793,7 +764,6 @@ class StarryDepot(object):
             return (ret, msg)
 
         self._setCache(key, value)
-        self.changes.add(key)
 
         return self._backup(key, check_flag=False)
 
@@ -851,7 +821,6 @@ class StarryDepot(object):
             return (ret, msg)
 
         self._setCache(key, value)
-        self.changes.add(key)
 
         return self._backup(key, check_flag=False)
 
@@ -892,7 +861,6 @@ class StarryDepot(object):
 
         if len(branch) == 0:
             self._deleteCache(key)
-            self.changes.add(key)
 
             try:
                 file.with_suffix(".mpb.tmp").unlink()
@@ -942,7 +910,6 @@ class StarryDepot(object):
             return (ret, msg)
 
         self._setCache(key, value)
-        self.changes.add(key)
 
         return self._backup(key, check_flag=False)
 
@@ -1113,7 +1080,10 @@ class StarryDepot(object):
             self.thread_cond.notify_all()
 
 class StarryClient(object):
-    def __init__(self, path, mode=LOCK_SH, address=("localhost", PORT)):
+    def __init__(self, path, mode=LOCK_SH, address=None):
+        if address is None:
+            address = msgpack.unpackb(pathlib.Path(os.path.join(path, ".starry")).read_bytes(), use_list=False)
+
         self.client = multiprocessing.connection.Client(address, authkey=b'starry')
         self.client.send((os.path.abspath(path), mode))
 
@@ -1129,6 +1099,12 @@ class StarryClient(object):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.__del__()
 
+    def close(self):
+        try:
+            self.client.close()
+        except:
+            pass
+
     def _callableForward(self, func, *args):
         try:
             self.client.send((func, *args))
@@ -1155,8 +1131,8 @@ class StarryClient(object):
     def delete(self, key):
         return self._callableForward("delete", key)
 
-    def clone(self, old_key, new_key):
-        return self._callableForward("clone", old_key, new_key)
+    def copy(self, old_key, new_key):
+        return self._callableForward("copy", old_key, new_key)
 
     def rename(self, old_key, new_key):
         return self._callableForward("rename", old_key, new_key)
@@ -1176,14 +1152,8 @@ class StarryClient(object):
     def recover(self):
         return self._callableForward("recover")
 
-    def save(self, path):
-        return self._callableForward("save", path)
-
-    def load(self, path):
-        return self._callableForward("load", path)
-
-    def replace(self, path):
-        return self._callableForward("replace", path)
+    def clone(self, path):
+        return self._callableForward("clone", path)
 
     def upgrade(self, path):
         return self._callableForward("upgrade", path)
@@ -1191,9 +1161,6 @@ class StarryClient(object):
     def clear(self):
         return self._callableForward("clear")
 
-    def destroy(self):
-        return self._callableForward("destroy")
-
     def getMultiList(self, keys):
         return self._callableForward("getMultiList", keys)
 
